package provide widgetLib 1.0

namespace eval widgetLib {
namespace export processOptions makeOptionList

################################################################
# proc makeOptionList {argList validSubwidgets \
#                      subwidgetVarName optionVarName }
#    Split an argument list into a list of subwidgets and options
#
#    Only include valid subwidgets (those in the validSubwidgets list)
#
#    If no valid subwidgets exist in argList, set the variable defined
#     by subwidgetVarName to the list  $validSubwidgets 
#
# Arguments
#   argList:	The argument list ?sub1? ?subX? -option1 ?val1? ?-option2 val2?
#   argList:	The argument list 
#               ?sub1? ?subX? -option1 ?val1? ?-option2 val2? ...
#
#   validSubwidgets:	A list of subwidget names that are valid for
#			the parent mega-widget
#
#   subwidgetVarName:	The name of the variable in the parent to receive the
#               list of subwidgets
#
#   optionVarName:	The name of the variable in the parent to receive the
#               list of options
# Results
#   Changes the values of the variables defined by $subwidgetVarName
#      and $optionVarName in the parent procedure.
# 
    proc makeOptionList {argList validSubwidgets \
                         subwidgetVarName optionVarName } {
        
	# Upvar the pass-by-reference variables

        upvar $subwidgetVarName subwidgetList
	upvar $optionVarName optionList
	
	# Look for an element that starts with  a -.  
	# All elements before the first  -option are subwidgets,
	#  and all elements after that are options or option/value pairs
	# If the subwidget does not include a ".", prepend the name with a
	# dot.

	set optStart [lsearch $argList "-*"]
	if {$optStart < 0} {return}
	foreach sub [lrange $argList 0 [expr $optStart -1]] {
	    if {[lsearch $validSubwidgets $sub] >= 0} {
	        lappend subwidgets $sub
	    }
	} 

	set optionList [lrange $argList $optStart end]
	
	# If there were no subwidgets in the arg list, then return a list
	#  of all valid subwidgets, with dots prepended as necessary.

	if {![info exists subwidgetList]} {
	    set subwidgets $validSubwidgets 
	}

	foreach name $subwidgets {
	        if {[string first "." $name] != 0} {
   	           lappend subwidgetList .${name}
		} else {
  	           lappend subwidgetList ${name}
		}
   	}
    }

################################################################
# proc processOptions {optionlist widgetlist {parent {}} }
#    Sets the options in the option list in each of the widgets in the
#    widget list.  
#
#    If parent is defined, then it is the parent of the widgets, and
#    it will pre-pended to the widget names.  
#
#    Expects widget names to start with "."
# Arguments
#   optionlist:	A list of options in the form: {-opt1 val1 -opt2 val2...}
#   widgetlist: A list of widgets to which these options will be applied.
#   parent:	The parent widget for the widgets in the widgetlist
# 
# Results
#   Changes the configuration of the widgets.
# 
    proc processOptions {optionlist widgetlist {parent {}} } {
        
	set return ""
	
	# Loop through the options and widgets.
	# If an option is not valid for a widget, fail silently 
	#    and continue processing

        foreach {option value} $optionlist {
	    foreach widget $widgetlist {
	        if {[string length $value] > 0} {
	            lappend return \
		        [catch "$parent$widget configure $option $value"]
		} else {
		    catch "$parent$widget configure $option " val
		    lappend return $val
		}
	    }
	}
        return $return
    }
}

